#
# This module defines the following variables:
#
#   ODB_USE_FILE - Path to the UseODB.cmake file. Use it to include the ODB use file.
#                  The use file defines the needed functionality to compile and use
#                  odb generated headers.
#
#   ODB_FOUND - All required components and the core library were found
#   ODB_INCLUDE_DIRS - Combined list of all components include dirs
#   ODB_LIBRARIES - Combined list of all componenets libraries, each prefixed with either "optimized" or "debug"
#   ODB_DEBUG_LIBRARIES - Combined list of all debug libraries, without prefixes
#   ODB_RELASE_LIBRARIES - Combined list of all release libraries, without prefixes
#
#   ODB_LIBODB_FOUND - Libodb core library was found
#   ODB_LIBODB_INCLUDE_DIRS - Include dirs for libodb core library
#   ODB_LIBODB_LIBRARIES - Libraries for libodb core library each prefixed with either "optimized" or "debug"
#
# For each requested component the following variables are defined:
#
#   ODB_<component>_FOUND - The component was found
#   ODB_<component>_INCLUDE_DIRS - The components include dirs
#   ODB_<component>_LIBRARIES - The components libraries
#
# <component> is the original or uppercase name of the component
#
# The component names relate directly to the odb module names.
# So for the libodb-mysql.so library, the component is named mysql,
# for the libodb-qt.so module it's qt, and so on.
#

set(ODB_USE_FILE "${CMAKE_CURRENT_LIST_DIR}/UseODB.cmake")

find_package(PkgConfig QUIET)

function(find_odb_api component)
    string(TOUPPER "${component}" component_u)
    set(ODB_${component_u}_FOUND FALSE PARENT_SCOPE)

    pkg_check_modules(PC_ODB_${component} QUIET "libodb-${component}")

    find_path(ODB_${component}_INCLUDE_DIR
            NAMES odb/${component}/version.hxx
            HINTS
            ${ODB_INCLUDE_DIRS}
            ${ODB_${component}_PATH}
            ${PC_ODB_${component}_INCLUDE_DIRS})

    find_library(ODB_${component}_LIBRARY_RELEASE
            NAMES odb-${component} libodb-${ODB_LIB_NAME}
            HINTS
            ${ODB_LIBODB_LIBRARIES}
            ${ODB_${component}_PATH}/lib
            ${ODB_${component}_PATH}/lib64
            ${PC_ODB_${component}_LIBRARY_DIRS})


    if (MSVC)
        find_library(ODB_${component}_LIBRARY_DEBUG
                NAMES odb-${component}-d libodb-${ODB_LIB_NAME}-d
                HINTS
                ${ODB_LIBODB_LIBRARIES}
                ${ODB_${component}_PATH}/lib
                ${ODB_${component}_PATH}/lib64
                ${PC_ODB_${component}_LIBRARY_DIRS})
    endif (MSVC)

    set(ODB_${component_u}_INCLUDE_DIRS ${ODB_${component}_INCLUDE_DIR})
    if (MSVC)
        set(ODB_${component_u}_LIBRARIES optimized ${ODB_${component}_LIBRARY_RELEASE} debug ${ODB_${component}_LIBRARY_DEBUG})
    else (MSVC)
        set(ODB_${component_u}_LIBRARIES ${ODB_${component}_LIBRARY_RELEASE})
    endif (MSVC)


    mark_as_advanced(ODB_${component}_INCLUDE_DIR ODB_${component}_LIBRARY)

    if (ODB_${component_u}_INCLUDE_DIRS AND ODB_${component_u}_LIBRARIES)
        set(ODB_${component_u}_FOUND TRUE PARENT_SCOPE)
        set(ODB_${component}_FOUND TRUE PARENT_SCOPE)

        list(APPEND ODB_INCLUDE_DIRS ${ODB_${component_u}_INCLUDE_DIRS})
        list(REMOVE_DUPLICATES ODB_INCLUDE_DIRS)
        set(ODB_INCLUDE_DIRS ${ODB_INCLUDE_DIRS} PARENT_SCOPE)

        list(APPEND ODB_LIBRARIES ${ODB_${component_u}_LIBRARIES})
        #		list(REMOVE_DUPLICATES ODB_LIBRARIES)
        set(ODB_LIBRARIES ${ODB_LIBRARIES} PARENT_SCOPE)

        if (ODB_DEBUG_LIBRARIES)
            list(APPEND ODB_DEBUG_LIBRARIES ${ODB_${component}_LIBRARY_DEBUG})
            list(REMOVE_DUPLICATES ODB_DEBUG_LIBRARIES)
            set(ODB_DEBUG_LIBRARIES ${ODB_DEBUG_LIBRARIES} PARENT_SCOPE)
        endif (ODB_DEBUG_LIBRARIES)

        list(APPEND ODB_RELEASE_LIBRARIES ${ODB_${component}_LIBRARY_RELEASE})
        list(REMOVE_DUPLICATES ODB_RELEASE_LIBRARIES)
        set(ODB_RELEASE_LIBRARIES ${ODB_RELEASE_LIBRARIES} PARENT_SCOPE)
    endif ()
endfunction()

pkg_check_modules(PC_LIBODB QUIET "libodb")

find_path(libodb_INCLUDE_DIR
        NAMES odb/version.hxx
        HINTS
        ${ODB_LIBODB_PATH}
        ${PC_LIBODB_INCLUDE_DIRS})

find_library(libodb_LIBRARY_RELEASE
        NAMES odb libodb
        HINTS
        ${ODB_LIBODB_PATH}/lib
        ${ODB_LIBODB_PATH}/lib64
        ${PC_LIBODB_LIBRARY_DIRS})

if (MSVC)
    find_library(libodb_LIBRARY_DEBUG
            NAMES odb-d libodb-d
            HINTS
            ${ODB_LIBODB_PATH}/lib
            ${ODB_LIBODB_PATH}/lib64
            ${PC_LIBODB_LIBRARY_DIRS})
endif (MSVC)

find_program(odb_BIN
        NAMES odb
        HINTS ${ODB_PATH}/bin ${libodb_INCLUDE_DIR}/../bin $ENV{ODB_PATH}/bin)

set(ODB_LIBODB_INCLUDE_DIRS ${libodb_INCLUDE_DIR})
if (MSVC)
    SET(ODB_LIBODB_LIBRARIES optimized ${libodb_LIBRARY_RELEASE} debug ${libodb_LIBRARY_DEBUG})
else (MSVC)
    SET(ODB_LIBODB_LIBRARIES ${libodb_LIBRARY_RELEASE})
endif (MSVC)
set(ODB_EXECUTABLE ${odb_BIN})

mark_as_advanced(libodb_INCLUDE_DIR libodb_LIBRARY odb_BIN)

if (ODB_LIBODB_INCLUDE_DIRS AND ODB_LIBODB_LIBRARIES)
    set(ODB_LIBODB_FOUND TRUE)
endif ()

set(ODB_INCLUDE_DIRS ${ODB_LIBODB_INCLUDE_DIRS})
set(ODB_LIBRARIES ${ODB_LIBODB_LIBRARIES})
set(ODB_DEBUG_LIBRARIES ${libodb_LIBRARY_DEBUG})
set(ODB_RELEASE_LIBRARIES ${libodb_LIBRARY_RELEASE})

foreach (component ${ODB_FIND_COMPONENTS})
    find_odb_api(${component})
endforeach ()

include(FindPackageHandleStandardArgs)
find_package_handle_standard_args(ODB
        FOUND_VAR ODB_FOUND
        REQUIRED_VARS ODB_EXECUTABLE ODB_LIBODB_FOUND
        HANDLE_COMPONENTS)
